###################################
#  Code to replicate the Figure 1 (Power curves) of the web appendix in simulations without covariates
###################################


##################################################
rm(list=ls())
require("orthopolynom")
require("polynom")
require(tmvtnorm)
require(MASS)
require(sfsmisc)
require(isotone)
require(fdrtool)
require(R2Cuba)
require(matlab)
library(snowfall)
library(RationalExp)


##### some useful functions
distor <- function(X,f){ return(  f(X)- mean(f(X)) + mean(X)   )}
scale <- function(X){return(rho*X)}

extr <-function(ls){
  Res = 0 
  for(i in 1:length(ls)){
    Res = Res + ls[[i]]
  }
  return(Res/length(ls))
}

quantile_extr <-function(ls,q){
  Res = ls[[1]]
  n_b <- length(Res)
  for(j in 1: n_b){
    cur = NULL
    for(i in 1:length(ls)){
      temp <- ls[[i]]
      cur <- c(cur, temp[j])
    }
    Res[j] <- quantile( cur,q,na.rm=T)
  }
  return(Res)
}
##########################################################
##########################################################



### discretisation pace for rho, sigma is fixed
nR <- 10
sigma_v <- seq(0.1,0.9,length.out =30)
nS <- length(sigma_v)
rho_v <-sigma_v 


#### number of cores for the computation
nbcores= 4

#### output matrices 
Result0_list<-  matrix(0,nS,nR)  # for out test
T_N0_list<-  matrix(0,nS,nR)
c_star0_list<-  matrix(0,nS,nR)
p_value0_list<-  matrix(0,nS,nR) 
Result_list<-  vector("list")
T_N_list<-  vector("list")
c_star_list<- vector("list")
p_value_list<-  vector("list")


level=0.05
### different number of sample points
n_list <-c(400,800,1200,1600,3200)
nMC=800
set.seed(1829384)


for(j in 1:length(n_list)){
  for(i in 1:length(rho_v)){ 
    n_p=n_list[j]
    n_y=n_p
    N <- n_y + n_p
    
    
   
    Result_list<-  vector("list")
    T_N_list<-  vector("list")
    c_star_list<- vector("list")
    Dev1<- vector("list")
    Dev2<- vector("list")
    Dev<- vector("list")
    # i=18
    rho <- rho_v[i]
    

    scale <- function(X){return(rho*X)}
    
    ### monte Carlo simulations
    for (i_mc in 1:nMC){
      
      ########################################################################################
  
        sig=0.1
        u=1
        b=0.10
        a=2
        
        ### Data generating process
        psi <-rnorm(n_p,0,u)
        pp_y <- runif(n_y,0,1)
        zeta <- rnorm(n_y,a,sig)
        zeta1 <- rnorm(n_y,-a,sig)
        pp1_y <- 1*(pp_y <b)
        pp2_y <- 1*(pp_y >1-b)
        pp3_y <- 1*(pp_y <=(1-b) & pp_y >=b)
        psi_y <-rnorm(n_y,0,u)
        y =  rho*psi_y+ pp1_y*zeta + pp2_y*zeta1    
         
        D <- rbind(matrix(1,n_y,1),matrix(0,n_p,1))
        W <-  N*((1-D)/sum((1-D)) - D / sum(D))
        y_tilde <- rbind(matrix(y,n_y,1),matrix(psi,n_p,1))
        X_tilde1 <- rbind(matrix(1,n_y,1),matrix(1,n_p,1))
        
        
        #### Parameters of the test
        prec =30
        Y_sup <- quantile(y_tilde,seq(0,1,length.out=prec))
        tuningParam<- vector(mode="list", length=6)
        tuningParam[["p"]] <- 0.05 # the parameter p in Section 3 in DGM
        tuningParam[["epsilon"]] <- 0.05 # 
        tuningParam[["B"]] <-500 #the number of bootstrap samples
        tuningParam[["y_grid"]] <-  Y_sup 
        tuningParam[["c"]] <- 0.3 # the parameter c in Section 3 in DGM
        tuningParam[["kappa"]] <- 0.001 #the parameter kappa in Section 3 in DGM
        
        ### test
        res <- test(y_tilde ,D,X_tilde1,NULL,NULL,nbcores,tuningParam )
        
  
        N <- res[[1]]
        cv01 <-   res[[2]]
        cv05 <-  res[[3]]
        cv10 <-  res[[4]]
        T_n<-  res[[5]]
        p_value <- res[[7]]
        T_reps <- res[[8]]
        Result <- !(T_n > cv05)
        Result_list[[i_mc]]<- Result
      }
      #### aggregation
      
      ### aggregate the results for all MonteCarlo simulations for these values of rho and of sample size N
      Result0_list[i,j]<- extr(Result_list)
      cat(paste("processing :  i", i, " j : ", j, " res: " ,Result0_list[i,j]  , "\n"))
  }
}


########## to plot the results
x11()
plot(sigma_v,1-Result0_list[,1], ylim=c(0,1), type="l", lwd=3, xlim=c(min(sigma_v),max(sigma_v)) ,font.axis=2,cex.axis=1.9)
par(new=TRUE)
plot(sigma_v,1-Result0_list[,2], ylim=c(0,1), type="l", lty=2, lwd=3, xlim=c(min(sigma_v),max(sigma_v)) ,font.axis=2,cex.axis=1.9)
par(new=TRUE)
plot(sigma_v,1-Result0_list[,3], ylim=c(0,1), type="l", lty=3, lwd=3, xlim=c(min(sigma_v),max(sigma_v)) ,font.axis=2,cex.axis=1.9)
par(new=TRUE)
plot(sigma_v,1-Result0_list[,4], ylim=c(0,1), type="l", lty=4, lwd=3, xlim=c(min(sigma_v),max(sigma_v)) ,font.axis=2,cex.axis=1.9)
par(new=TRUE)
plot(sigma_v,1-Result0_list[,5], ylim=c(0,1), type="l", lty=5, lwd=3, xlim=c(min(sigma_v),max(sigma_v)) ,font.axis=2,cex.axis=1.9)
abline(v=0.616,lwd=1)
abline(h=0.05,lwd=1,lty="dotted")
legend(0.65, 0.8, legend=c("n= 400", "n= 800", "n= 1, 200","n= 1, 600","n= 3, 200"),
       lwd=3,   lty=c(1:5), cex=1.2)



