###################################
#  Code to replicate the descriptive statistics, Table 1
###################################

rm(list=ls())

## load packages
library(foreign)
require(pracma)
require(MESS)
require(sfsmisc)
library(ggplot2)
library(reshape2)
library(matlab)
library(Hmisc)

path="TO COMPLETE WITH THE PATH OF THE DATA FOLDER"
#### Labor market data
p_year ="sce-labor-microdata-public_2017.csv"
#### Labor market data
p_year2 ="FRBNY-SCE-Public-Microdata-Complete.csv"


#######################################   Labor market module    ################################################################## 
data_third <- read.csv2(paste0(path,p_year),  header =TRUE, sep=";", dec=",")
names <-colnames(data_third)
data_p <- as.data.frame(data_third)

#### Variable selection 
selection <- c("date", "userid","l9","l10","l3","oo2e2","oo2e2dk_1")
sub_data <- data_p[,selection]
inc_grid <- c(10000,20000,30000,40000,50000,60000,75000,100000,150000,200000)
past_inc <- inc_grid[sub_data$l10]
sub_data$past_inc <-past_inc
sub_data <- sub_data[,c("date", "userid", "past_inc","l9","l3","oo2e2","oo2e2dk_1")]
add_2 <- sub_data$l3
add_3 <- sub_data$l9
sub_data$past_inc <-add_2
sub_data$past_inc1 <-add_3
sub_data_p <- sub_data[,c("date", "userid", "past_inc","past_inc1","oo2e2","oo2e2dk_1")]

## for anticipated earnings
sub_data_exp <-subset(sub_data_p ,select=c("date", "userid","oo2e2","oo2e2dk_1"))
## for realized earnings
sub_data_real <-subset(sub_data_p, select=c("date", "userid","past_inc"))
sub_data_real <- na.omit(sub_data_real)

#######################################   global SCE survey      ##################################################################

### import global SCE survey
data_third <- read.csv2(paste0(path,p_year2),  header =TRUE, sep=";", dec=",")
data_p <- as.data.frame(data_third[-c(1),])
colnames(data_p)<- as.matrix(data_third[c(1),])

## Variable selection
selection <- c("date", "userid", "weight","Q10_1","Q10_2","Q10_3","Q10_4","Q23v2part2","Q24_bin1" ,"Q24_bin2" ,
               "Q24_bin3" ,"Q24_bin4", "Q24_bin5", "Q24_bin6", "Q24_bin7", "Q24_bin8", "Q24_bin9", "Q24_bin10",
               "Q32","Q47","Q37","Q33","Q35_1","Q35_2","Q12new","Q36","Q34","Q36","Q34", "Q37", "Q38", "Q46","_EDU_CAT","_NUM_CAT","_REGION_CAT")
sub_data <- data_p[,selection]
sub_data$Q12new
sub_data$Q24_bin1[1]
inc_grid <- c(10000,15000,25000,35000,45000,55000,67500,87500,125000,175000,200000)
past_inc <- inc_grid[sub_data$Q47]
work1 <- sub_data[,"Q10_1"]
work1<- as.numeric(as.vector(work1))
work2 <- sub_data[,"Q10_2"]
work2<- as.numeric(as.vector(work2))
work <- work1+work2
work <- 1*(work>=1)
sub_data$work <-work
unemp1 <- sub_data[,"Q10_3"]
unemp1<- as.numeric(as.vector(unemp1))
unemp2 <- sub_data[,"Q10_4"]
unemp2<- as.numeric(as.vector(unemp2))
unemp <- unemp1+unemp2
unemp <- 1*(unemp>=1)
sub_data$unemp <-unemp
sub_data$past_inc <-past_inc
colnames(sub_data)
not_self <- sub_data$Q12new
not_self <- as.numeric(as.vector(not_self ))
not_self[is.na(not_self)] <-0
self <- (not_self==2)*1
sub_data$self <-self
sub_data$sex <- as.numeric(sub_data$Q33)-2
sub_data$white1 <- as.numeric(sub_data$Q35_1)-1
sub_data$black <- as.numeric(sub_data$Q35_2)-1
sub_data$hisp <- ((as.numeric(sub_data$Q34)-1)==1)*1

sub_data$white <-  sub_data$white1 & !sub_data$hisp
sub_data$NUM_CAT <- as.numeric(sub_data[,"_NUM_CAT"]) -1
sub_data$exp <- as.numeric(sub_data[,"Q37"])-1


sub_data$lessHS <- (as.numeric(sub_data$Q36)-1)==1 |(as.numeric(sub_data$Q36)-1)==2 |(as.numeric(sub_data$Q36)-1)==3 
sub_data$HS <- (as.numeric(sub_data$Q36)-1)>=4

sub_data <- sub_data[,c("date", "userid", "weight","Q32",'work','past_inc',"Q37","unemp","self","sex","white","black", "lessHS","HS","exp","NUM_CAT")]
colnames(sub_data)<- c("date", "userid", "weight","age",'work','past_inc',"duration","unemp","self","sex","white","black", "lessHS","HS","exp","NUM_CAT")
weights <- as.character(sub_data[,3])
weights <- as.numeric(gsub(",", ".", weights))
sub_data$weight <- weights
sub_data <- as.data.frame(sub_data)
sub_data2 <-NULL
for (j in 1:dim(sub_data)[2]){
  sub_data2 <-cbind(sub_data2,  as.numeric(as.vector(sub_data[,j])))
}
sub_data <- as.data.frame(sub_data2)
colnames(sub_data)<- c("date", "userid", "weight","age",'work','past_inc',"duration","unemp","self","sex","white","black", "lessHS","HS","exp","NUM_CAT")
sub_data <- sub_data[,c("date", "userid", "weight","age",'work',"unemp","self","sex","white","black", "lessHS","HS","exp","NUM_CAT")]
colnames(sub_data)<- c("date", "userid", "weight","age",'work',"unemp","self","sex","white","black", "lessHS","HS","exp","NUM_CAT")
sub_data <- na.omit(sub_data)
dates <- as.numeric(as.vector(unique(sub_data$date)))

sub_dataW <- sub_data[,c("date", "userid", "weight","sex","white","black", "lessHS","HS","exp","NUM_CAT")]
colnames(sub_dataW)<- c("date", "userid", "weight","sex","white","black", "lessHS","HS","exp","NUM_CAT")

##########################  Merge Labor Module with Main SCE    ##################################
data_1 <- merge(sub_data_exp,sub_data, by="userid")
data_1_sel <- data_1[data_1$date.x==data_1$date.y,]
colnames(data_1_sel)[2] <- "date"

data_2 <- merge(sub_data_real,sub_data, by="userid")
data_2_sel <- data_2[data_2$date.x==data_2$date.y,]
colnames(data_2_sel)[2] <- "date"
indic <- data_1_sel$oo2e2dk_1==1 & data_1_sel$work==0 & data_1_sel$unemp==0
indic1 <- indic
indic1[is.na(indic)] <- FALSE
data_1_sel1 <- data_1_sel[!indic1,]
data_1_sel1$oo2e2[data_1_sel1$oo2e2dk_1==1] <- 0
data_1_sel <- subset(data_1_sel1,select=c( "userid" ,  "date", "oo2e2", "weight" ,"age", "work" ,"unemp", "self","sex","white","black", "lessHS","HS",
                                           "exp","NUM_CAT"))
data_1_sel <- na.omit(data_1_sel)

### Remove unemployed
indic <- data_1_sel$work==0 & data_1_sel$unemp==0  
indic1 <- indic
indic1[is.na(indic)] <- FALSE
data_1_sel <- data_1_sel[!indic1,]

#######################################################################################
##################   save all samples                        ###############

pathout="TO COMPLETE WITH THE PATH OF THE CODES FOLDER"
save(sub_data_real, file=paste0(pathout,"sub_data_real.Rdata"))
save(sub_data_exp, file=paste0(pathout,"sub_data_exp.Rdata"))
save(data_1_sel, file=paste0(pathout,"data_1_sel.Rdata"))
save(data_2_sel, file=paste0(pathout,"data_2_sel.Rdata"))
save(sub_data, file=paste0(pathout,"sub_data.Rdata"))


